/*==============================================================================
 * Пример перенаправления функций ввода/вывода printf() и scanf() через UART,
 * бегущими светодиодами и сканированием кнопок на плате.
 *------------------------------------------------------------------------------
 * НИИЭТ, Богдан Колбов <kolbov@niiet.ru>
 * НИИЭТ, Александр Дыхно <dykhno@niiet.ru>
 *==============================================================================
 * ДАННОЕ ПРОГРАММНОЕ ОБЕСПЕЧЕНИЕ ПРЕДОСТАВЛЯЕТСЯ «КАК ЕСТЬ», БЕЗ КАКИХ-ЛИБО
 * ГАРАНТИЙ, ЯВНО ВЫРАЖЕННЫХ ИЛИ ПОДРАЗУМЕВАЕМЫХ, ВКЛЮЧАЯ ГАРАНТИИ ТОВАРНОЙ
 * ПРИГОДНОСТИ, СООТВЕТСТВИЯ ПО ЕГО КОНКРЕТНОМУ НАЗНАЧЕНИЮ И ОТСУТСТВИЯ
 * НАРУШЕНИЙ, НО НЕ ОГРАНИЧИВАЯСЬ ИМИ. ДАННОЕ ПРОГРАММНОЕ ОБЕСПЕЧЕНИЕ
 * ПРЕДНАЗНАЧЕНО ДЛЯ ОЗНАКОМИТЕЛЬНЫХ ЦЕЛЕЙ И НАПРАВЛЕНО ТОЛЬКО НА
 * ПРЕДОСТАВЛЕНИЕ ДОПОЛНИТЕЛЬНОЙ ИНФОРМАЦИИ О ПРОДУКТЕ, С ЦЕЛЬЮ СОХРАНИТЬ ВРЕМЯ
 * ПОТРЕБИТЕЛЮ. НИ В КАКОМ СЛУЧАЕ АВТОРЫ ИЛИ ПРАВООБЛАДАТЕЛИ НЕ НЕСУТ
 * ОТВЕТСТВЕННОСТИ ПО КАКИМ-ЛИБО ИСКАМ, ЗА ПРЯМОЙ ИЛИ КОСВЕННЫЙ УЩЕРБ, ИЛИ
 * ПО ИНЫМ ТРЕБОВАНИЯМ, ВОЗНИКШИМ ИЗ-ЗА ИСПОЛЬЗОВАНИЯ ПРОГРАММНОГО ОБЕСПЕЧЕНИЯ
 * ИЛИ ИНЫХ ДЕЙСТВИЙ С ПРОГРАММНЫМ ОБЕСПЕЧЕНИЕМ.
 *
 *                              2018 АО "НИИЭТ"
 *==============================================================================
 */

//-- Includes ------------------------------------------------------------------
#include "system_K1921VK01T.h"
#include "K1921VK01T.h"
#include "bsp.h"
#include "retarget_conf.h"


//-- Defines -------------------------------------------------------------------
uint32_t MODE = 0;
static volatile uint32_t leds = 0x01;

//--Functions ------------------------------------------------------------------
void Blinky(void);



//-- Types ---------------------------------------------------------------------
typedef enum {
    COMMON_REG_SYS_CLK_SEL_SRC_CPE = 0x0,      /*!< select POR clock or XI_OSC by input CPE */
    COMMON_REG_SYS_CLK_SEL_SRC_POR = 0x1,      /*!< POR clock */
    COMMON_REG_SYS_CLK_SEL_SRC_ExtOsc = 0x2,   /*!<  external oscillator clock (XI_OSC) */
    COMMON_REG_SYS_CLK_SEL_SRC_PLL = 0x3,      /*!< PLL clock */
    COMMON_REG_SYS_CLK_SEL_SRC_PLLDiv = 0x4,   /*!< PLL clock with post-divider */
    COMMON_REG_SYS_CLK_SEL_SRC_USB60MHz = 0x5, /*!< USB clock 60 MHz */
    COMMON_REG_SYS_CLK_SEL_SRC_USBClk = 0x6,   /*!< clock from input CLK_USB GPIO A0 (12/24 MHz) */
    COMMON_REG_SYS_CLK_SEL_SRC_ETH25MHz = 0x7, /*!< clock from external Ethernet clock GPIO B3 or GPIO A8 */
} COMMON_REG_SYS_CLK_SEL_SRC_Enum;

typedef enum {
    COMMON_REG_PLL_CTRL_REFSEL_ExtOsc = 0x0,   /*!< tact signal from external oscillator */
    COMMON_REG_PLL_CTRL_REFSEL_USBClk = 0x1,   /*!< tact signal from USB clock(GPIO A0) */
    COMMON_REG_PLL_CTRL_REFSEL_USB60MHz = 0x2, /*!< tact signal from USB 60 MHz */
    COMMON_REG_PLL_CTRL_REFSEL_ETH25MHz = 0x3, /*!< tact signal from Ethernet 25 MHz */
} COMMON_REG_PLL_CTRL_REFSEL_Enum;

//-- Peripheral init functions -------------------------------------------------
void PLL_Init()
{
    NT_COMMON_REG->SYS_CLK = COMMON_REG_SYS_CLK_SEL_SRC_ExtOsc;
    uint32_t pll_stable_wait = 1000;
    while (pll_stable_wait) {
        pll_stable_wait--;
    }    
        NT_COMMON_REG->PLL_CTRL_bit.REFSEL = COMMON_REG_PLL_CTRL_REFSEL_ExtOsc;
    NT_COMMON_REG->PLL_OD = 1;
    NT_COMMON_REG->PLL_NF = 164; //Для кварца 10МГц, Системная частота = 69,16МГц
//    NT_COMMON_REG->PLL_NF = 140; //Для кварца 12МГц, Системная частота = 71МГц    
    NT_COMMON_REG->PLL_NR = 4;
    NT_COMMON_REG->PLL_CTRL_bit.PLL_DIV = 0;
        pll_stable_wait = 1000;
    while (pll_stable_wait) {
        pll_stable_wait--;
    }
     NT_BOOTFLASH->T_ACC = 6;
     NT_COMMON_REG->SYS_CLK = COMMON_REG_SYS_CLK_SEL_SRC_PLLDiv;
} 








//-- Peripheral init functions -------------------------------------------------
void periph_init()
{
	  PLL_Init();
//    retarget_init();
    BSP_LED_Init();
    BSP_Btn_Init();
    SysTick_Config(10000000);
}

//-- Main ----------------------------------------------------------------------
int main()
{
    uint32_t i,tmp;
//    char string[256];

    periph_init();
//    printf("K1921VK01T> All periph inited\n");
//    printf("K1921VK01T> CPU frequency is %.3f MHz\n", SystemCoreClock / 1E6);
//    printf("K1921VK01T> Start LED run ...\n");
//    printf("K1921VK01T> Enter Any String ...\n");
//    printf("K1921VK01T> Your string: %s\n",string);
//    printf("K1921VK01T> Press Any Key ...\n");
    while (1) {
			  tmp = BSP_Btn_GetKey();
			  while(tmp == NOKEY) 
				{
					if(MODE != 0)
					  leds = 0x01;
					
					MODE = 0;
					tmp = BSP_Btn_GetKey();
				}
		
				if (tmp == MULTIPLE) 
				{
//					printf("Pressed Some Keys\n");
				}
        else  
          for (i=1;i<NUM_BTN_CODES-1;i++) 
            if (tmp == i) 
            {
//						  printf("Pressed Key %d\n",i);
              
              if(MODE != i)
              {								
						    MODE = i; 
						    leds = 0x01;
							}
						}
    };
}

//-- IRQ handlers --------------------------------------------------------------
void SysTick_Handler()
{
	Blinky();
}

void Blinky(void)
{
	switch(MODE)
	{
		case 0:
			if(leds == (0xFF))
			{
				leds = 0x01;
			}
			else
				leds++;
		break;
			
		case 1:
			if(leds == 0xFF)
				leds = 0x00;
			else
				leds = 0xFF;
		break;
			
		case 2:
			if(leds == 0x01 || leds == 0x18)
				leds = 0x81;
			else
				leds = (leds & 0xF0)>>1 | (leds & 0x0F)<<1;
		break;
		
		case 3:
			if(leds == 0x01 || leds == 0x81)
				leds = 0x18;
			else
				leds = (leds & 0xF0)<<1 | (leds & 0x0F)>>1;
		break;
			
		case 4:
			if(leds == (0x01))
			{
				leds = 0x80;
			}
			else
				leds = leds >> 1;
		break;
		
		case 5:
			if(leds == (1<<7))
			{
				leds = 0x01;
			}
			else
				leds = leds << 1;
		break;
	}
	
	BSP_LED_On(leds);
	BSP_LED_Off(~leds & 0xFF);
}
